library(forecast)
library(ggplot2)
library(data.table)

# Datenherkunft:
# Eigenes Archiv von https://www.katholisch.at/site/kirche/sl/kircheinoesterreich/statistik
# Carsten Frerk, Christoph Baumgarten: Gottes Werk und unser Beitrag
# Fowid: https://fowid.de/meldung/oesterreich-kirchenstatistik-katholische-kirche-1991-2018
# Statistik Austria, Bevölkerung zu Jahresbeginn
# Statistik Austria, Verbraucherpreisindex 2000

kk = read.csv2("kk-daten.csv", dec = ",")
kk$bevölkerungsanteil = kk$katholiken / kk$gesamtbevölkerung
kk$medianbeitrag = kk$medianeinkommen * 0.011
# 5-Jahres-Mittelwert der Austritte
kk$austritte5j = frollmean(kk$austritte, 5, fill=mean(kk$austritte[1:4]))
# Austrittsrate: Austritte des Jahres verglichen mit dem Stand des Vorjahres
kk$austrittsrate_rk = kk$austritte / (kk$austritte + kk$katholiken)
# 5-Jahres-Mittelwert der römisch-katholischen Austrittsrate
kk$austrittsrate5j = frollmean(kk$austrittsrate_rk, 5, fill=mean(kk$austrittsrate_rk[1:4]))
# Kumulierte Summe der Austritte seit Beginn der Daten
kk$austritte_kum = cumsum(kk$austritte)

standardplot = function (daten) {
    ggplot(daten) +
        theme_grey() + 
        geom_line(linewidth = 0.9)
}

standardplot(kk) +
    aes(x = jahr, y = katholiken) +
    geom_line() +
    labs(
        x = "Jahr", y = "Katholiken am Jahresende",
        title = "Katholiken in Österreich am Jahresende"
    ) +
    scale_y_continuous(limits = c(0, max(kk$katholiken)), labels = scales::label_number(big.mark = ".", decimal.mark = ","))

ggsave("katholiken_am_jahresende.png", plot = last_plot(), device = png(), 
        width = 1600, height = 1000, units = "px")
dev.off()

standardplot(kk) +
    aes(x = jahr, y = austritte) +
    labs(
        x = "Jahr", y = "Austritte",
        title = "Austritte aus der röm.-katholischen Kirche"
    ) +
    scale_y_continuous(limits = c(0, max(kk$austritte)), labels = scales::label_number(big.mark = ".", decimal.mark = ","))

ggsave("austritte_pro_jahr.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

ggplot(kk) +
    aes(x = jahr) +
    geom_line(aes(y=austrittsrate_rk), colour = "darkblue", linewidth = 0.6) +
    geom_line(aes(y=austrittsrate_ab), colour = "orange", linewidth = 0.6) +
    labs(
        x = NULL, y = "Austrittsrate",
        title = "Austrittsrate aus der röm.-katholischen und der evangelischen (AB) Kirche"
    ) +
    theme_grey() +
    scale_y_continuous(limits = c(0, max(kk$austrittsrate_ab)), labels = scales::label_percent(big.mark = ".", decimal.mark = ","))+
    theme(plot.title = element_text(size=9))

ggsave("austrittsrate_rk_eab.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Zeitreihe aus den Austritten
austritts = ts(kk$austrittsrate_rk, start=min(kk$jahr))
# ETS-Forecast aus den Austritten
fcaustritte = ets(austritts, "ZAN")
# Forecast für die nächsten Jahre ausgeben
forecast(fcaustritte)
autoplot(forecast(fcaustritte)) + 
    labs(title = "Vorhersage (ETS) der Austrittsrate aus der römisch-katholischen Kirche", 
         x = NULL, y = NULL) +
    scale_y_continuous(limits = c(-0.02, 0.05), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ","))+
    theme(plot.title = element_text(size=9))
ggsave("vorhersage_ets_austritte.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Zeitreihe aus den Katholiken
kaths = ts(kk$katholiken, start = min(kk$jahr))
# Forecast für die Katholiken
fckath = ets(kaths)
forecast(fckath)
autoplot(forecast(fckath)) + 
    labs(title = "Vorhersage (ETS) für Katholiken in Österreich") +
    scale_y_continuous(limits = c(0, 6500000), 
                       labels = scales::label_number(big.mark = ".", decimal.mark = ","))

ggsave("vorhersage_ets_katholiken.png", plot = last_plot(), device = png(), 
       width = 1600, height = 800, units = "px")

dev.off()

# Zeitreihe aus dem Bevölkerungsanteil der Katholiken
ants = ts(kk$katholiken / kk$gesamtbevölkerung, start = min(kk$jahr))
# Forecast für den Bevölkerungsanteil
fcants = ets(ants)
forecast(fcants)
autoplot(forecast(fcants)) + 
    labs(title = "Vorhersage (ETS) für Katholiken-Anteil an der Bevölkerung", 
         x=NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 1), labels = scales::label_percent()) +
    theme(plot.title = element_text(size=12))
antforecastdiag

ggsave("vorhersage_ets_anteil_katholiken.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Modell: Austrittsrate erklärt durch die Jahreszahl
mjahr = lm(austrittsrate_rk ~ jahr, data = kk)
mjahr
anova(mjahr)
summary(mjahr)
kk$vorhersage_austrittsrate_jahr = mjahr$fitted.values

ggplot(kk) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = vorhersage_austrittsrate_jahr), colour = "red", linewidth = 0.6, linetype = 2) +
    labs(title = "Austrittsrate und Vorhersage mit Jahreszahl", x = NULL, y = "Austrittsrate") +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
    ) 

ggsave("vorhersage_austrittsrate_jahr.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()


# Daten von 1997 bis 2022 (wo das Medianeinkommen zur Verfügung steht)
kk97_22 = kk[7:32,]

# Für 2022 Median-Beitrag mit forecast vorhergesagt
mb22 = forecast(ets(ts(kk$medianbeitrag[7:31], start = 1997)), h = 1)$mean[1]
kk97_22$medianbeitrag = c(kk97_22$medianbeitrag[1:25], mb22)

# Modell: Austrittssrate erklärt durch den Median-Kirchenbeitrag
mmkb = lm(austrittsrate_rk ~ medianbeitrag, data = kk97_22)
# Korrelation zwischen den Vorhersagen und den Austritten
summary(mmkb)

kk97_22$erg_medianbeitrag = mmkb$fitted.values

ggplot(kk97_22) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = erg_medianbeitrag), colour = "red", linewidth = 0.6, linetype = 2) +
    labs(title = "Austrittsrate und Vorhersage mit Median-Kirchenbeitrag", x = NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
                       ) +
    theme(plot.title = element_text(size=10))

ggsave("vorhersage_austrittsrate_medianbeitrag.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Modell: Austritte erklärt durch Nicht-Katholikenanteil im Vorjahr
kk$bevanteilvj = c(NA, 1- kk$bevölkerungsanteil[1:31])
kk$austrittevj = c(NA, kk$austritte[1:31])
mank = lm(austrittsrate_rk ~ bevanteilvj, data = kk)
summary(mank)
kk$erg_säkularisierung = c(NA, mank$fitted.values)

ggplot(kk) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = erg_säkularisierung), colour = "red", linewidth = 0.6, linetype = 2) +
    labs(title = "Austrittsrate und Vorhersage mit Grad der Säkularisierung", x = NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
    ) +
    theme(plot.title = element_text(size=10))

ggsave("vorhersage_austrittsrate_saekul.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Modell: Austritte erklärt durch kumulierte Summe der Austritte
mkum = lm(austrittsrate_rk ~ austritte_kum, data = kk)
summary(mkum)
kk$erg_kumsum = mkum$fitted.values

ggplot(kk) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = erg_kumsum), colour = "red", linewidth = 0.6, linetype = 2) +
    labs(title = "Austrittsrate und Vorhersage mit kumulierter Summe der Austritte", x = NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
    ) +
    theme(plot.title = element_text(size=10))

ggsave("vorhersage_austrittsrate_kumsum.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Modell: Austritte erklärt durch Nicht-Katholikenanteil und Austritte im Vorjahr
kk$bevanteilvj = c(NA, 1- kk$bevölkerungsanteil[1:31])
kk$austrittevj = c(NA, kk$austritte[1:31])
manv = lm(austrittsrate_rk ~ bevanteilvj + austrittevj, data = kk)
summary(manv)
kk$erg_säkularisierung_austritte = c(NA, manv$fitted.values)

ggplot(kk) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = erg_säkularisierung_austritte), colour = "red", linewidth = 0.6, linetype = 2) +
    labs(title = "Austrittsrate und Vorhersage mit Grad der Säkularisierung und Vorjahres-Austritten", x = NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
    ) +
    theme(plot.title = element_text(size=9))

ggsave("vorhersage_austrittsrate_saekul_austr.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Modell: Austritte erklärt durch Säkularisierung und Skandale
mss = lm(austrittsrate_rk ~ bevanteilvj + skandal, data = kk)
summary(mss)
kk$erg_ss = c(NA, mss$fitted.values)

ggplot(kk) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = erg_ss), colour = "red", linewidth = 0.6, linetype=2) +
    labs(title = "Vorhersage der Austrittsrate mit Säkularisierung und Skandalen", x = NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
    ) +
    theme(plot.title = element_text(size=10))

ggsave("vorhersage_austrittsrate_saekul_skandale.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

# Modell: Austritte erklärt durch Medianeinkommen, Skandale
mmbs = lm(austrittsrate_rk ~ medianbeitrag + skandal, data = kk97_22)
summary(mmbs)
kk97_22$erg_mbs = mmbs$fitted.values

ggplot(kk97_22) + aes(x = jahr) +
    geom_line(aes(y = austrittsrate_rk), colour = "darkgreen", linewidth = 0.6) +
    geom_line(aes(y = erg_mbs), colour = "red" ,linewidth = 0.6, linetype=2) +
    labs(title = "Vorhersage der Austrittsrate mit Medianbeitrag und Skandalen", x = NULL, y = NULL) +
    scale_y_continuous(limits = c(0, 0.02), 
                       labels = scales::label_percent(big.mark = ".", decimal.mark = ",")
    ) +
    theme(plot.title = element_text(size=10))

ggsave("vorhersage_austrittsrate_medianeink_skandale.png", plot = last_plot(), device = png(), 
       width = 1600, height = 1000, units = "px")
dev.off()

bavj = 1 - kk$bevölkerungsanteil[length(kk$bevölkerungsanteil)]

# Vorhersage für 2023
# Annahme: Skandal-Wert = 0,4
j2023 = data.frame(bevanteilvj = c(bavj), skandal = c(0.4))
predict(mss, j2023)
predict(mss, j2023) * kk$katholiken[length(kk$katholiken)]

# Annahme: Skandal-Wert = 0,7
j2023 = data.frame(bevanteilvj = c(bavj), skandal = c(0.7))
predict(mss, j2023)
predict(mss, j2023) * kk$katholiken[length(kk$katholiken)]
